import * as i0 from '@angular/core';
import { Injectable, Inject, Directive, ElementRef, EventEmitter, Input, Output, ContentChild, HostListener, NgModule } from '@angular/core';
import { Subject, fromEvent, merge, timer } from 'rxjs';
import { withLatestFrom, filter, mapTo, takeUntil, throttleTime, takeWhile } from 'rxjs/operators';
import * as i1 from '@angular/common';
import { DOCUMENT } from '@angular/common';

class NgsgElementsHelper {
    static findIndex(element) {
        const allElements = element.parentNode.children;
        return Array.prototype.indexOf.call(allElements, element);
    }
}

class ScrollHelperService {
    constructor(document) {
        this.document = document;
        this.DEFAULT_SCROLLSPEED = 50;
        this.SCROLL_BUFFER = 50;
        this.window = document.defaultView;
    }
    scrollIfNecessary(event, scrollPoints = {}, scrollSpeed) {
        const currentPosition = event.pageY - this.window.scrollY;
        if (this.isTopScrollNeeded(currentPosition, scrollPoints.top)) {
            this.window.scrollBy({ top: -scrollSpeed || -this.DEFAULT_SCROLLSPEED, behavior: 'smooth' });
            return;
        }
        if (this.isBottomScrollNeeded(currentPosition, scrollPoints.bottom)) {
            this.window.scrollBy({ top: scrollSpeed || this.DEFAULT_SCROLLSPEED, behavior: 'smooth' });
        }
    }
    isTopScrollNeeded(currentPosition, scrollPointTop) {
        return scrollPointTop ? currentPosition < scrollPointTop :
            currentPosition < this.SCROLL_BUFFER;
    }
    isBottomScrollNeeded(currentPosition, scrollPointBottom) {
        return scrollPointBottom ? currentPosition > scrollPointBottom :
            currentPosition > this.window.innerHeight - this.SCROLL_BUFFER;
    }
}
/** @nocollapse */ ScrollHelperService.ɵprov = i0.ɵɵdefineInjectable({ factory: function ScrollHelperService_Factory() { return new ScrollHelperService(i0.ɵɵinject(i1.DOCUMENT)); }, token: ScrollHelperService, providedIn: "root" });
/** @type {!Array<{type: !Function, args: (undefined|!Array<?>)}>} */
ScrollHelperService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/**
 * @type {function(): !Array<(null|{
 *   type: ?,
 *   decorators: (undefined|!Array<{type: !Function, args: (undefined|!Array<?>)}>),
 * })>}
 * @nocollapse
 */
ScrollHelperService.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] }] }
];

class NgsgClassService {
    constructor() {
        this.SELECTED_DEFAULT_CLASS = 'ng-sg-selected';
        this.PLACEHOLDER_DEFAULT_CLASS = 'ng-sg-placeholder';
        this.DROPPED_DEFAULT_CLASS = 'ng-sg-dropped';
        this.ACTIVE_DEFAULT_CLASS = 'ng-sg-active';
    }
    addPlaceHolderClass(element) {
        element.classList.add(this.PLACEHOLDER_DEFAULT_CLASS);
    }
    removePlaceHolderClass(element) {
        element.classList.remove(this.PLACEHOLDER_DEFAULT_CLASS);
    }
    addDroppedClass(element) {
        element.classList.add(this.DROPPED_DEFAULT_CLASS);
    }
    removeDroppedClass(element) {
        element.classList.remove(this.DROPPED_DEFAULT_CLASS);
    }
    addSelectedClass(element) {
        element.classList.add(this.SELECTED_DEFAULT_CLASS);
    }
    removeSelectedClass(element) {
        element.classList.remove(this.SELECTED_DEFAULT_CLASS);
    }
    addActiveClass(element) {
        element.classList.add(this.ACTIVE_DEFAULT_CLASS);
    }
    removeActiveClass(element) {
        element.classList.remove(this.ACTIVE_DEFAULT_CLASS);
    }
}
/** @nocollapse */ NgsgClassService.ɵprov = i0.ɵɵdefineInjectable({ factory: function NgsgClassService_Factory() { return new NgsgClassService(); }, token: NgsgClassService, providedIn: "root" });
/** @type {!Array<{type: !Function, args: (undefined|!Array<?>)}>} */
NgsgClassService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];

class NgsgStoreService {
    constructor() {
        this.state = new Map();
    }
    initState(group, items = [], classes = {}) {
        this.state.set(group, { items: [...items], classes, selectedItems: [] });
    }
    addSelectedItem(group, dragElement) {
        this.state.get(group).selectedItems.push(dragElement);
    }
    removeSelectedItem(group, item) {
        const updatedItems = this.state.get(group).selectedItems
            .filter((dragElement) => dragElement.node !== item);
        this.setSelectedItems(group, updatedItems);
    }
    setItems(group, items) {
        this.state.get(group).items = [...items];
    }
    getItems(group) {
        return this.state.get(group).items;
    }
    hasItems(group) {
        return this.getItems(group).length > 0;
    }
    hasGroup(group) {
        return this.state.has(group);
    }
    getSelectedItems(group) {
        return this.state.get(group).selectedItems;
    }
    setSelectedItems(group, selectedItems) {
        this.state.get(group).selectedItems = [...selectedItems];
    }
    getFirstSelectItem(group) {
        return this.state.get(group).selectedItems[0];
    }
    hasSelectedItems(group) {
        return this.getSelectedItems(group).length > 0;
    }
    resetSelectedItems(group) {
        this.setSelectedItems(group, []);
    }
    getClasses(group) {
        return this.state.get(group).classes;
    }
}
/** @nocollapse */ NgsgStoreService.ɵprov = i0.ɵɵdefineInjectable({ factory: function NgsgStoreService_Factory() { return new NgsgStoreService(); }, token: NgsgStoreService, providedIn: "root" });
/** @type {!Array<{type: !Function, args: (undefined|!Array<?>)}>} */
NgsgStoreService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];

var ChangeAction;
(function (ChangeAction) {
    ChangeAction[ChangeAction["ADD"] = 0] = "ADD";
    ChangeAction[ChangeAction["REMOVE"] = 1] = "REMOVE";
})(ChangeAction || (ChangeAction = {}));
class NgsgSelectionService {
    constructor(classService, ngsgStore) {
        this.classService = classService;
        this.ngsgStore = ngsgStore;
        this.COMMAND_KEY = 'Meta';
        this.CONTROL_KEY = 'Control';
        this.selectionChange$ = new Subject();
        const selectionKeyPressed$ = this.selectionKeyPressed();
        this.selectionChange$
            .pipe(withLatestFrom(selectionKeyPressed$))
            .subscribe(([selectionChange, selectionKeyPressed]) => {
            selectionKeyPressed
                ? this.handleSelectionChange(selectionChange)
                : this.resetSelectedItems(selectionChange.key);
        });
    }
    resetSelectedItems(group) {
        this.ngsgStore.getSelectedItems(group).forEach(item => this.classService.removeSelectedClass(item.node));
        this.ngsgStore.resetSelectedItems(group);
    }
    handleSelectionChange(selectionChange) {
        if (selectionChange.action === ChangeAction.ADD) {
            this.classService.addSelectedClass(selectionChange.item);
            this.ngsgStore.addSelectedItem(selectionChange.key, {
                node: selectionChange.item,
                originalIndex: NgsgElementsHelper.findIndex(selectionChange.item)
            });
        }
        if (selectionChange.action === ChangeAction.REMOVE) {
            this.classService.removeSelectedClass(selectionChange.item);
            this.ngsgStore.removeSelectedItem(selectionChange.key, selectionChange.item);
        }
    }
    selectionKeyPressed() {
        const selectionKeyPressed = fromEvent(window, 'keydown').pipe(filter((keyboardEvent) => keyboardEvent.key === this.COMMAND_KEY || keyboardEvent.key === this.CONTROL_KEY), mapTo(true));
        const keyup = fromEvent(window, 'keyup').pipe(mapTo(false));
        return merge(selectionKeyPressed, keyup);
    }
    selectElementIfNoSelection(group, dragedElement) {
        if (this.ngsgStore.hasSelectedItems(group)) {
            return;
        }
        this.ngsgStore.addSelectedItem(group, {
            node: dragedElement,
            originalIndex: NgsgElementsHelper.findIndex(dragedElement)
        });
    }
    updateSelectedDragItem(key, item, selected) {
        this.selectionChange$.next({
            key,
            item,
            action: selected ? ChangeAction.ADD : ChangeAction.REMOVE
        });
    }
}
/** @nocollapse */ NgsgSelectionService.ɵprov = i0.ɵɵdefineInjectable({ factory: function NgsgSelectionService_Factory() { return new NgsgSelectionService(i0.ɵɵinject(NgsgClassService), i0.ɵɵinject(NgsgStoreService)); }, token: NgsgSelectionService, providedIn: "root" });
/** @type {!Array<{type: !Function, args: (undefined|!Array<?>)}>} */
NgsgSelectionService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/**
 * @type {function(): !Array<(null|{
 *   type: ?,
 *   decorators: (undefined|!Array<{type: !Function, args: (undefined|!Array<?>)}>),
 * })>}
 * @nocollapse
 */
NgsgSelectionService.ctorParameters = () => [
    { type: NgsgClassService },
    { type: NgsgStoreService }
];

class NgsgEventsService {
    constructor() {
        this.dropped$ = new Subject();
    }
}
/** @nocollapse */ NgsgEventsService.ɵprov = i0.ɵɵdefineInjectable({ factory: function NgsgEventsService_Factory() { return new NgsgEventsService(); }, token: NgsgEventsService, providedIn: "root" });
/** @type {!Array<{type: !Function, args: (undefined|!Array<?>)}>} */
NgsgEventsService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];

class NgsgReflectService {
    constructor(ngsgStore) {
        this.ngsgStore = ngsgStore;
    }
    reflectChanges(key, element) {
        const items = this.ngsgStore.getItems(key);
        const selectedElements = this.ngsgStore.getSelectedItems(key);
        const selectedElementIndices = this.getSelectedElementsIndices(selectedElements);
        const selectedItems = this.getSelectedItems(items, selectedElementIndices);
        const sortedIndices = selectedElementIndices.sort();
        const dropIndex = this.findDropIndex(selectedElements, element);
        while (sortedIndices.length > 0) {
            items.splice(sortedIndices.pop(), 1);
        }
        const result = this.getReflectedItems(items, selectedItems, dropIndex);
        this.ngsgStore.setItems(key, result);
        return result;
    }
    getReflectedItems(items, selectedItems, dropIndex) {
        const beforeSelection = items.slice(0, dropIndex);
        const afterSelection = items.slice(dropIndex, items.length);
        return [...beforeSelection, ...selectedItems, ...afterSelection];
    }
    getSelectedItems(items, selectedElementIndexes) {
        const selectedItems = [];
        selectedElementIndexes.forEach(index => {
            selectedItems.push(items[index]);
        });
        return selectedItems;
    }
    getSelectedElementsIndices(selectedElements) {
        return selectedElements.map((selectedElement) => selectedElement.originalIndex);
    }
    findDropIndex(selectedElements, element) {
        if (this.isDropInSelection(selectedElements, element)) {
            return NgsgElementsHelper.findIndex(selectedElements[0].node);
        }
        return NgsgElementsHelper.findIndex(element);
    }
    isDropInSelection(collection, dropElement) {
        return !!collection.find((dragElment) => dragElment.node === dropElement);
    }
}
/** @nocollapse */ NgsgReflectService.ɵprov = i0.ɵɵdefineInjectable({ factory: function NgsgReflectService_Factory() { return new NgsgReflectService(i0.ɵɵinject(NgsgStoreService)); }, token: NgsgReflectService, providedIn: "root" });
/** @type {!Array<{type: !Function, args: (undefined|!Array<?>)}>} */
NgsgReflectService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/**
 * @type {function(): !Array<(null|{
 *   type: ?,
 *   decorators: (undefined|!Array<{type: !Function, args: (undefined|!Array<?>)}>),
 * })>}
 * @nocollapse
 */
NgsgReflectService.ctorParameters = () => [
    { type: NgsgStoreService }
];

class NgsgSortService {
    constructor(classService, ngsgStore) {
        this.classService = classService;
        this.ngsgStore = ngsgStore;
    }
    initSort(group) {
        this.dragIndex = this.ngsgStore.getFirstSelectItem(group).originalIndex;
        this.dragElements = this.ngsgStore.getSelectedItems(group);
    }
    sort(dropElement) {
        const hoverIndex = NgsgElementsHelper.findIndex(dropElement);
        const el = this.getSibling(dropElement, this.dragIndex, hoverIndex);
        if (this.isDropInSelection(el)) {
            return;
        }
        this.dragElements.forEach((dragElement) => {
            const insertedNode = dropElement.parentNode.insertBefore(dragElement.node, el.node);
            this.classService.addPlaceHolderClass(insertedNode);
        });
        this.dragIndex = NgsgElementsHelper.findIndex(this.dragElements[0].node);
    }
    endSort() {
        this.dragElements.forEach((dragElement) => {
            this.updateDropedItem(dragElement.node);
        });
    }
    getSibling(dropElement, dragIndex, hoverIndex) {
        if (dragIndex < hoverIndex) {
            return { node: dropElement.nextSibling, originalIndex: hoverIndex + 1 };
        }
        return { node: dropElement, originalIndex: hoverIndex };
    }
    isDropInSelection(dropElement) {
        return !!this.dragElements.find((dragElment) => dragElment.node === dropElement.node);
    }
    updateDropedItem(item) {
        this.classService.removePlaceHolderClass(item);
        this.classService.addDroppedClass(item);
        this.classService.removeSelectedClass(item);
        this.classService.removeActiveClass(item);
        timer(500).subscribe(() => this.classService.removeDroppedClass(item));
    }
}
/** @nocollapse */ NgsgSortService.ɵprov = i0.ɵɵdefineInjectable({ factory: function NgsgSortService_Factory() { return new NgsgSortService(i0.ɵɵinject(NgsgClassService), i0.ɵɵinject(NgsgStoreService)); }, token: NgsgSortService, providedIn: "root" });
/** @type {!Array<{type: !Function, args: (undefined|!Array<?>)}>} */
NgsgSortService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/**
 * @type {function(): !Array<(null|{
 *   type: ?,
 *   decorators: (undefined|!Array<{type: !Function, args: (undefined|!Array<?>)}>),
 * })>}
 * @nocollapse
 */
NgsgSortService.ctorParameters = () => [
    { type: NgsgClassService },
    { type: NgsgStoreService }
];

class NgsgDragHandleDirective {
    constructor(el) {
        this.el = el;
    }
}
/** @type {!Array<{type: !Function, args: (undefined|!Array<?>)}>} */
NgsgDragHandleDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ngSortgridDragHandle]'
            },] }
];
/**
 * @type {function(): !Array<(null|{
 *   type: ?,
 *   decorators: (undefined|!Array<{type: !Function, args: (undefined|!Array<?>)}>),
 * })>}
 * @nocollapse
 */
NgsgDragHandleDirective.ctorParameters = () => [
    { type: ElementRef }
];

const selector = '[ngSortgridItem]';
class NgsgItemDirective {
    constructor(el, sortService, selectionService, reflectService, ngsgStore, ngsgEventService, scrollHelperService, classService) {
        this.el = el;
        this.sortService = sortService;
        this.selectionService = selectionService;
        this.reflectService = reflectService;
        this.ngsgStore = ngsgStore;
        this.ngsgEventService = ngsgEventService;
        this.scrollHelperService = scrollHelperService;
        this.classService = classService;
        this.ngSortGridGroup = 'defaultGroup';
        this.autoScroll = false;
        this.sorted = new EventEmitter();
        this.selected = false;
        this.destroy$ = new Subject();
    }
    ngOnInit() {
        this.ngsgEventService.dropped$.pipe(takeUntil(this.destroy$)).subscribe(() => this.selected = false);
        fromEvent(this.el.nativeElement, 'drag').pipe(throttleTime(20), takeUntil(this.destroy$), takeWhile(() => this.autoScroll)).subscribe(() => {
            this.scrollHelperService.scrollIfNecessary(event, {
                top: this.scrollPointTop,
                bottom: this.scrollPointBottom
            }, this.scrollSpeed);
        });
    }
    ngOnChanges(changes) {
        const sortGridItemChanges = changes.ngSortGridItems;
        const sortGridItems = sortGridItemChanges.currentValue ? sortGridItemChanges.currentValue : [];
        if (!this.ngsgStore.hasGroup(this.ngSortGridGroup)) {
            this.ngsgStore.initState(this.ngSortGridGroup, sortGridItems);
            return;
        }
        this.ngsgStore.setItems(this.ngSortGridGroup, sortGridItems);
    }
    ngAfterViewInit() {
        var _a, _b;
        this.handleElement = ((_b = (_a = this.handle) === null || _a === void 0 ? void 0 : _a.el) === null || _b === void 0 ? void 0 : _b.nativeElement) || this.el.nativeElement;
        fromEvent(this.handleElement, 'mousedown').pipe(takeUntil(this.destroy$)).subscribe(() => {
            this.el.nativeElement.draggable = true;
        });
    }
    ngOnDestroy() {
        this.destroy$.next();
        this.destroy$.complete();
    }
    dragStart(event) {
        if (!this.occuredOnHost(event)) {
            return;
        }
        this.selectionService.selectElementIfNoSelection(this.ngSortGridGroup, event.target);
        this.classService.addActiveClass(event.target);
        this.sortService.initSort(this.ngSortGridGroup);
    }
    dragEnter() {
        if (!this.ngsgStore.hasSelectedItems(this.ngSortGridGroup)) {
            return;
        }
        this.sortService.sort(this.el.nativeElement);
    }
    dragOver(event) {
        if (event.preventDefault) {
            // Necessary. Allows us to drop.
            event.preventDefault();
        }
        return false;
    }
    drop() {
        this.el.nativeElement.draggable = false;
        if (!this.ngsgStore.hasSelectedItems(this.ngSortGridGroup)) {
            return;
        }
        if (!this.ngsgStore.hasItems(this.ngSortGridGroup)) {
            console.warn(`Ng-sortgrid: No items provided - please use [sortGridItems] to pass in an array of items -
      otherwhise the ordered items can not be emitted in the (sorted) event`);
            return;
        }
        const previousOrder = [...this.ngsgStore.getItems(this.ngSortGridGroup)];
        this.sortService.endSort();
        const currentOrder = this.reflectService.reflectChanges(this.ngSortGridGroup, this.el.nativeElement);
        this.sorted.next({ previousOrder, currentOrder });
        this.ngsgStore.resetSelectedItems(this.ngSortGridGroup);
        this.ngsgEventService.dropped$.next();
    }
    clicked() {
        this.selected = !this.isItemCurrentlySelected();
        this.selectionService.updateSelectedDragItem(this.ngSortGridGroup, this.el.nativeElement, this.selected);
    }
    isItemCurrentlySelected() {
        const index = NgsgElementsHelper.findIndex(this.el.nativeElement);
        return !!this.ngsgStore.getSelectedItems(this.ngSortGridGroup)
            .find(element => element.originalIndex === index);
    }
    occuredOnHost(event) {
        return event.target.matches(selector);
    }
}
/** @type {!Array<{type: !Function, args: (undefined|!Array<?>)}>} */
NgsgItemDirective.decorators = [
    { type: Directive, args: [{ selector },] }
];
/**
 * @type {function(): !Array<(null|{
 *   type: ?,
 *   decorators: (undefined|!Array<{type: !Function, args: (undefined|!Array<?>)}>),
 * })>}
 * @nocollapse
 */
NgsgItemDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: NgsgSortService },
    { type: NgsgSelectionService },
    { type: NgsgReflectService },
    { type: NgsgStoreService },
    { type: NgsgEventsService },
    { type: ScrollHelperService },
    { type: NgsgClassService }
];
/** @type {!Object<string, !Array<{type: !Function, args: (undefined|!Array<?>)}>>} */
NgsgItemDirective.propDecorators = {
    ngSortGridGroup: [{ type: Input }],
    ngSortGridItems: [{ type: Input }],
    scrollPointTop: [{ type: Input }],
    scrollPointBottom: [{ type: Input }],
    scrollSpeed: [{ type: Input }],
    autoScroll: [{ type: Input }],
    sorted: [{ type: Output }],
    handle: [{ type: ContentChild, args: [NgsgDragHandleDirective,] }],
    dragStart: [{ type: HostListener, args: ['dragstart', ['$event'],] }],
    dragEnter: [{ type: HostListener, args: ['dragenter',] }],
    dragOver: [{ type: HostListener, args: ['dragover', ['$event'],] }],
    drop: [{ type: HostListener, args: ['dragend',] }],
    clicked: [{ type: HostListener, args: ['click',] }]
};

class NgsgModule {
}
/** @type {!Array<{type: !Function, args: (undefined|!Array<?>)}>} */
NgsgModule.decorators = [
    { type: NgModule, args: [{
                declarations: [NgsgItemDirective, NgsgDragHandleDirective],
                exports: [NgsgItemDirective, NgsgDragHandleDirective]
            },] }
];

/*
 * Public API Surface of ng-sortgrid
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NgsgItemDirective, NgsgModule, NgsgDragHandleDirective as ɵa, NgsgSortService as ɵb, NgsgClassService as ɵc, NgsgStoreService as ɵd, NgsgSelectionService as ɵe, NgsgReflectService as ɵf, NgsgEventsService as ɵg, ScrollHelperService as ɵh };
//# sourceMappingURL=ng-sortgrid.js.map
